@@ -6,7 +6,7 @@ use EV ();
 use Carp ();
 use Socket ();
 
-our $VERSION = '1.506_02';
+our $VERSION = '1.506_03';
 
 require Feersum::Connection;
 require Feersum::Connection::Handle;
@@ -51,11 +51,13 @@ sub use_socket {
             if ($family == Socket::AF_INET()) {
                 (my $packed_port, my $packed_addr) = Socket::sockaddr_in($sockaddr);
                 $host = Socket::inet_ntoa($packed_addr) || 'localhost';
-                $port = $packed_port || 80;
+                # Use defined check - port 0 is valid (OS-assigned dynamic port)
+                $port = defined($packed_port) ? $packed_port : 80;
             } elsif (eval { Socket::AF_INET6() } && $family == Socket::AF_INET6()) {
                 (my $packed_port, my $packed_addr) = Socket::sockaddr_in6($sockaddr);
                 $host = Socket::inet_ntop(Socket::AF_INET6(), $packed_addr) || 'localhost';
-                $port = $packed_port || 80;
+                # Use defined check - port 0 is valid (OS-assigned dynamic port)
+                $port = defined($packed_port) ? $packed_port : 80;
             }
         }
     }
@@ -450,6 +452,25 @@ the tolerances provided by libev).  If an entity body is part of the request
 (e.g. POST or PUT) it will wait this long between successful C<read()> system
 calls.
 
+=item C<< header_timeout() >>
+
+=item C<< header_timeout($seconds) >>
+
+Get or set the header completion deadline timeout (Slowloris protection).
+Default is 0 (disabled).
+
+When enabled, connections must complete sending all HTTP headers within this
+many seconds from connection acceptance or receive a 408 Request Timeout
+response. This is a B<hard deadline> that does not reset when data arrives,
+unlike C<read_timeout> which resets on each successful read.
+
+This provides protection against Slowloris-style attacks where malicious
+clients send headers very slowly to exhaust server connection resources.
+
+Recommended value for direct internet exposure: 30-60 seconds. When running
+behind a reverse proxy (nginx, HAProxy), this can typically be left disabled
+since the proxy handles slow clients.
+
 =item C<< graceful_shutdown(sub { .... }) >>
 
 Causes Feersum to initiate a graceful shutdown of all outstanding connections.
