@@ -48,6 +48,8 @@ sub new { ## no critic (RequireArgUnpacking)
 sub DESTROY {
     local $@;
     my $self = shift;
+    return if $self->{_destroyed};  # Guard against double-DESTROY
+    $self->{_destroyed} = 1;
     if (my $f = $self->{endjinn}) {
         $f->request_handler(sub{});
         $f->unlisten();
@@ -67,11 +69,15 @@ sub _create_socket {
             unlink $listen or carp "unlink stale socket '$listen': $!";
         }
         my $saved = umask(0);
-        $sock = IO::Socket::UNIX->new(
-           Local => rel2abs($listen),
-           Listen => SOMAXCONN,
-        );
-        umask($saved);
+        $sock = eval {
+            IO::Socket::UNIX->new(
+               Local => rel2abs($listen),
+               Listen => SOMAXCONN,
+            );
+        };
+        my $err = $@;
+        umask($saved);  # Restore umask even if socket creation failed
+        die $err if $err;
         croak "couldn't bind to socket: $!" unless $sock;
         $sock->blocking(0) || croak "couldn't unblock socket: $!";
     }
@@ -88,7 +94,14 @@ sub _create_socket {
                 # IPv6 without port: [::1]
                 ($host, $port, $is_ipv6) = ($1, 0, 1);
             } elsif ($listen =~ /:.*:/) {
-                # Bare IPv6 (multiple colons): ::1 or 2001:db8::1
+                # Bare IPv6 (multiple colons): could be ::1 or 2001:db8::1
+                # Reject ambiguous cases - require bracket notation for IPv6 with ports
+                # e.g., ::1:8080 is ambiguous (address ::1 port 8080? or address ::1:8080?)
+                if ($listen =~ /:(\d{1,5})$/ && $1 <= 65535) {
+                    # Looks like it ends with a port number - ambiguous!
+                    croak "ambiguous IPv6 address '$listen': use bracket notation [host]:port " .
+                          "(e.g., [::1]:$1 or [2001:db8::1]:$1)";
+                }
                 ($host, $port, $is_ipv6) = ($listen, 0, 1);
             } else {
                 # IPv4: host:port
@@ -98,6 +111,11 @@ sub _create_socket {
                 $is_ipv6 = 0;
             }
 
+            # Validate port range (0-65535)
+            if ($port !~ /^\d+$/ || $port > 65535) {
+                croak "invalid port '$port': must be 0-65535";
+            }
+
             my ($domain, $sockaddr);
             if ($is_ipv6) {
                 defined AF_INET6()
@@ -132,6 +150,14 @@ sub _create_socket {
                 || do { close($sock); croak "couldn't unblock socket: $!"; };
         }
         else {
+            # Validate port in listen address for better error messages
+            if ($listen =~ /:(\d+)$/) {
+                my $port = $1;
+                croak "invalid port '$port': must be 0-65535" if $port > 65535;
+            } elsif ($listen =~ /:(\S+)$/) {
+                my $port = $1;
+                croak "invalid port '$port': must be numeric" unless $port =~ /^\d+$/;
+            }
             $sock = IO::Socket::INET->new(
                 LocalAddr => $listen,
                 ReuseAddr => 1,
@@ -156,6 +182,8 @@ sub _prepare {
         [ ($self->{host}||DEFAULT_HOST).':'.($self->{port}||DEFAULT_PORT) ];
     croak "listen must be an array reference"
         if ref $self->{listen} ne 'ARRAY';
+    croak "listen array cannot be empty"
+        if @{$self->{listen}} == 0;
     croak "Feersum doesn't support multiple 'listen' directives yet"
         if @{$self->{listen}} > 1;
     my $listen = shift @{$self->{listen}};
@@ -163,8 +191,23 @@ sub _prepare {
 
     if (my $opts = $self->{options}) {
         $self->{$_} = delete $opts->{$_} for grep defined($opts->{$_}),
-            qw/pre_fork keepalive read_timeout max_connection_reqs reuseport epoll_exclusive
+            qw/pre_fork keepalive read_timeout header_timeout max_connection_reqs reuseport epoll_exclusive
                read_priority write_priority accept_priority max_accept_per_loop max_connections/;
+        # Warn about unknown options (likely typos)
+        for my $unknown (keys %$opts) {
+            carp "Unknown option '$unknown' ignored";
+        }
+    }
+
+    # Validate pre_fork early (before socket creation) to fail fast
+    if ($self->{pre_fork}) {
+        my $n = $self->{pre_fork};
+        if ($n !~ /^\d+$/ || $n < 1) {
+            croak "pre_fork must be a positive integer";
+        }
+        if ($n > MAX_PRE_FORK) {
+            croak "pre_fork=$n exceeds maximum of " . MAX_PRE_FORK;
+        }
     }
 
     # Enable reuseport automatically in prefork mode if SO_REUSEPORT available
@@ -186,6 +229,7 @@ sub _prepare {
 
     $f->set_keepalive($_) for grep defined, delete $self->{keepalive};
     $f->read_timeout($_) for grep defined, delete $self->{read_timeout};
+    $f->header_timeout($_) for grep defined, delete $self->{header_timeout};
     $f->max_connection_reqs($_) for grep defined, delete $self->{max_connection_reqs};
     # Validate priority values (-2 to +2 per libev)
     for my $prio_name (qw/read_priority write_priority accept_priority/) {
@@ -268,7 +312,10 @@ sub _fork_another {
         # This eliminates accept() contention for better scaling
         if ($self->{_use_reuseport}) {
             $self->{endjinn}->unlisten();
-            close($self->{sock}) or carp "close socket: $!" if $self->{sock};
+            if ($self->{sock}) {
+                close($self->{sock})
+                    or croak "close parent socket in child: $! (fd leak risk)";
+            }
             my $sock = $self->_create_socket($self->{_listen_addr}, 1);
             $self->{sock} = $sock;
             $self->{endjinn}->use_socket($sock);
@@ -315,14 +362,7 @@ sub _fork_another {
 sub _start_pre_fork {
     my $self = shift;
 
-    # Validate pre_fork value
-    my $n = $self->{pre_fork};
-    if (!defined $n || $n !~ /^\d+$/ || $n < 1) {
-        croak "pre_fork must be a positive integer";
-    }
-    if ($n > MAX_PRE_FORK) {
-        croak "pre_fork=$n exceeds maximum of " . MAX_PRE_FORK;
-    }
+    # pre_fork value already validated in _prepare()
 
     POSIX::setsid() or carp "setsid() failed: $!";
 
@@ -335,9 +375,13 @@ sub _start_pre_fork {
 
     # With SO_REUSEPORT, parent can close its socket entirely
     # Children have their own sockets
-    if ($self->{_use_reuseport}) {
-        close($self->{sock}) or carp "close socket: $!" if $self->{sock};
-        $self->{sock} = undef;
+    if ($self->{_use_reuseport} && $self->{sock}) {
+        if (close($self->{sock})) {
+            $self->{sock} = undef;
+        } else {
+            carp "close socket: $!";
+            # Keep reference so destructor can retry cleanup
+        }
     }
     return;
 }
